import Flutter
import UIKit
import ComPDFKit
import ComPDFKit_Tools

public class CompdfkitFlutterPlugin: NSObject, FlutterPlugin, CPDFViewBaseControllerDelete {
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "com.compdfkit.flutter.plugin", binaryMessenger: registrar.messenger())
        let instance = CompdfkitFlutterPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "sdk_version_code":
            result(CPDFKit.sharedInstance().versionString)
        case "init_sdk":
            let initInfo = call.arguments as? [String: Any]
            let key = initInfo?["key"] ?? ""
            CPDFKit.verify(withKey: key as? String)
        case "sdk_build_tag":
            result("iOS build tag:\(CPDFKit.sharedInstance().buildNumber)")
        case "openDocument":
            let initInfo = call.arguments as? [String: Any]
            let jsonString = initInfo?["configuration"] ?? ""
            _ = initInfo?["password"] ?? ""
            let path = initInfo?["document"] as? String ?? ""
            let jsonDic = readJSON(jsonString as! String)
            let configuration = parseJSON(jsonDic)
            
            if let rootViewControl = UIApplication.shared.keyWindow?.rootViewController {
                var tRootViewControl = rootViewControl
                
                if let presentedViewController = rootViewControl.presentedViewController {
                    tRootViewControl = presentedViewController
                }
                
                let pdfViewController = CPDFViewController(filePath: path, password: nil, configuration: configuration)
                let navController = CNavigationController(rootViewController: pdfViewController)
                pdfViewController.delegate = self
                navController.modalPresentationStyle = .fullScreen
                tRootViewControl.present(navController, animated: true)
            }
        case "getTemporaryDirectory":
            result(self.getTemporaryDirectory())
            
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
    // MARK: - Private Methods
    
    func readJSON(_ jsonFilePath: String) -> Dictionary<String, Any> {
        do {
            if let jsonData = jsonFilePath.data(using: .utf8) {
                
                let jsonDic = try JSONSerialization.jsonObject(with: jsonData, options: JSONSerialization.ReadingOptions.mutableContainers) as? [String: Any] ?? [String: Any]()
                
                return jsonDic
            }
            
        } catch{
            
        }
        
        return Dictionary()
    }
    
    func parseJSON(_ jsonDic: Dictionary<String, Any>) -> CPDFConfiguration {
        let configuration = CPDFConfiguration()
        
        for (key, value) in jsonDic {
            if key == "modeConfig" {
                if let innerDict = value as? [String: Any] {
                    for (innerKey, innerValue) in innerDict {
                        if innerKey == "initialViewMode" {
                            let innerValueString = innerValue as? String ?? ""
                            
                            if innerValueString == "viewer" {
                                configuration.enterToolModel = .viewer
                            } else if innerValueString == "annotations" {
                                configuration.enterToolModel = .annotation
                            } else if innerValueString == "contentEditor" {
                                configuration.enterToolModel = .edit
                            } else if innerValueString == "forms" {
                                configuration.enterToolModel = .form
                            } else if innerValueString == "digitalSignatures" {
                                configuration.enterToolModel = .signature
                            }
                        } else if innerKey == "availableViewModes" {
                            if let innerArray = innerValue as? [Any] {
                                var availableViewModes: [CPDFToolFunctionTypeState] = []
                                for (_, item) in innerArray.enumerated() {
                                    let itemString = item as? String ?? ""
                                    
                                    if itemString == "viewer" {
                                        availableViewModes.append(.viewer)
                                    } else if itemString == "annotations" {
                                        availableViewModes.append(.annotation)
                                    } else if itemString == "contentEditor" {
                                        availableViewModes.append(.edit)
                                    } else if itemString == "forms" {
                                        availableViewModes.append(.form)
                                    } else if itemString == "digitalSignatures" {
                                        availableViewModes.append(.signature)
                                    }
                                }
                                configuration.availableViewModes = availableViewModes
                            }
                        }
                    }
                }
            } else if key == "toolbarConfig" {
                if let innerDict = value as? [String: Any] {
                    for (innerKey, innerValue) in innerDict {
                        if let innerArray = innerValue as? [Any] {
                            if innerKey == "iosRightBarAvailableActions" {
                                var showRightItems: [CNavBarButtonItem] = []
                                for (_, item) in innerArray.enumerated() {
                                    let itemString = item as? String ?? ""
                                    
                                    if itemString == "search" {
                                        let search = CNavBarButtonItem(viewRightBarButtonItem: .search)
                                        showRightItems.append(search)
                                    } else if itemString == "bota" {
                                        let bota = CNavBarButtonItem(viewRightBarButtonItem: .bota)
                                        showRightItems.append(bota)
                                    } else if itemString == "menu" {
                                        let more = CNavBarButtonItem(viewRightBarButtonItem: .more)
                                        showRightItems.append(more)
                                    }
                                }
                                configuration.showRightItems = showRightItems
                            } else if innerKey == "iosLeftBarAvailableActions" {
                                var showleftItems: [CNavBarButtonItem] = []
                                for (_, item) in innerArray.enumerated() {
                                    let itemString = item as? String ?? ""
                                    
                                    if itemString == "back" {
                                        let back = CNavBarButtonItem(viewLeftBarButtonItem: .back)
                                        showleftItems.append(back)
                                    } else if itemString == "thumbnail" {
                                        let thumbnail = CNavBarButtonItem(viewLeftBarButtonItem: .thumbnail)
                                        showleftItems.append(thumbnail)
                                    }
                                }
                                configuration.showleftItems = showleftItems
                            } else if innerKey == "availableMenus" {
                                var showMoreItems: [CPDFPopMenuViewType] = []
                                for (_, item) in innerArray.enumerated() {
                                    let itemString = item as? String ?? ""
                                    
                                    if itemString == "viewSettings" {
                                        showMoreItems.append(.setting)
                                    } else if itemString == "documentEditor" {
                                        showMoreItems.append(.pageEdit)
                                    } else if itemString == "security" {
                                        showMoreItems.append(.security)
                                    } else if itemString == "watermark" {
                                        showMoreItems.append(.watermark)
                                    } else if itemString == "documentInfo" {
                                        showMoreItems.append(.info)
                                    } else if itemString == "save" {
                                        showMoreItems.append(.save)
                                    } else if itemString == "share" {
                                        showMoreItems.append(.share)
                                    } else if itemString == "openDocument" {
                                        showMoreItems.append(.addFile)
                                    }
                                }
                                configuration.showMoreItems = showMoreItems
                            }
                        }
                    }
                }
            }
        }
        
        return configuration
    }
    
    
    func getTemporaryDirectory() -> String {
        let paths = NSSearchPathForDirectoriesInDomains(.cachesDirectory, .userDomainMask, true)
        return paths.first ?? ""
    }
    
    // MARK: - CPDFViewBaseControllerDelete
    
    public func PDFViewBaseControllerDissmiss(_ baseControllerDelete: CPDFViewBaseController) {
        baseControllerDelete.dismiss(animated: true)
    }
    
}
